/**
 * Represents an action that can be performed by the optimizer.
 * All optimizations must be done as an action. This provides
 * a central point for creating logs and reports of the work
 * that was done. It can also be used to implement backtracking.
 */
import { SourcePosition } from "@opticss/element-analysis";
import { Keys } from "@opticss/util";
import * as postcss from "postcss";
import { Optimizations } from "../optimizations";
export declare abstract class Action {
    optimization: string;
    abstract perform(): this;
    abstract logString(): string;
    abstract readonly sourcePosition: SourcePosition | undefined | null;
    constructor(optimization: Keys<Optimizations>);
    positionString(sourcePosition?: SourcePosition | null | undefined, includeFilename?: boolean): string;
    annotateLogMessage(message: string, sourcePosition?: SourcePosition | null, indent?: number): string;
    nodeSourcePosition(node: postcss.Node): {
        filename: string;
        line: number;
        column: number;
    } | undefined;
}
export declare abstract class MultiAction extends Action {
    logString(): string;
    abstract logStrings(): Array<string>;
}
export declare function isMultiAction(action: Action): action is MultiAction;
export declare function stripNL(str: string): string;
