import { SourcePosition } from "@opticss/element-analysis";
import { SimpleAttribute as ElementAttribute, SimpleTagname as ElementTagname, StyleMapping, TemplateIntegrationOptions } from "@opticss/template-api";
import * as postcss from "postcss";
import { Optimizations } from "../../OpticssOptions";
import { OptimizationPass } from "../../OptimizationPass";
import { DeclarationInfo } from "../../optimizations/MergeDeclarations/StyleInfo";
import { CompoundSelector, ParsedSelector } from "../../parseSelector";
import { ParsedSelectorAndRule, SelectorCache } from "../../query";
import { IdentGenerators } from "../../util/IdentGenerator";
import { MultiAction } from "../Action";
export interface Declaration {
    prop: string;
    value: string;
    important: boolean;
}
/**
 * Merges duplicate declarations from multiple rule sets into a new rule set.
 */
export declare class MergeDeclarations extends MultiAction {
    private templateOptions;
    removedSelectors: ParsedSelectorAndRule[];
    selectorContext: ParsedSelector | undefined;
    removedAtRules: postcss.AtRule[];
    styleMapping: StyleMapping;
    removedRules: postcss.Rule[];
    declInfos: DeclarationInfo[];
    decl: Declaration;
    newRule: postcss.Rule;
    identGenerators: IdentGenerators<"id" | "class">;
    container: postcss.Container;
    reason: string;
    cache: SelectorCache;
    constructor(templateOptions: TemplateIntegrationOptions, pass: OptimizationPass, selectorContext: ParsedSelector | undefined, decl: Declaration, declInfos: Array<DeclarationInfo>, optimization: keyof Optimizations, reason: string);
    perform(): this;
    logStrings(): Array<string>;
    declString(selector?: string): string;
    readonly sourcePosition: SourcePosition | undefined;
    /**
     * Returns the concrete html traits that the selector would match
     * or undefined if html traits aren't deducible from the selector.
     */
    static inputsFromSelector(templateOptions: TemplateIntegrationOptions, sel: CompoundSelector): Array<ElementTagname | ElementAttribute> | undefined;
}
