import { Attr, AttributeValue } from "@opticss/element-analysis";
import * as SelectorParser from "postcss-selector-parser";
import { Match } from "./Match";
import { HasSelectorNodes, Matcher } from "./Matcher";
export declare function isAttrNode(node: SelectorParser.Node): node is SelectorParser.Attribute;
export declare class AttributeMatcher extends Matcher<Attr> {
    private constructor();
    private static _instance;
    static readonly instance: AttributeMatcher;
    matchSelectorComponent(attr: Attr, selector: HasSelectorNodes): Match;
    matchSelectorNode(attr: Attr, node: SelectorParser.Node): Match;
    /**
     * Match an attribute against a single string value. This is used
     * by the id selector and some attribute selectors.
     *
     * @param identifier The identifier in the selector to match against the
     *   element's attribute value.
     * @param [value=this.value] the attribute value to match against.
     * @returns boolean if it matches or not
     */
    matchIdent(attr: Attr, identifier: string, value?: AttributeValue, whitespaceDelimited?: boolean): Match;
    /**
     * Match an attribute against an attribute value using space delimited
     * matching semantics. This is used by the class selector and some attribute
     * selectors.
     * @param identifier The identifier in the selector to match against the
     *   element's attribute value.
     * @param [value=this.value] the attribute value to match against.
     * @returns boolean if it matches
     */
    matchWhitespaceDelimited(attr: Attr, identifier: string, value?: AttributeValue): boolean;
    matchAttributeNode(attr: Attr, node: SelectorParser.Node): Match;
    regexForAttrEq(attr: Attr, caseInsensitive: boolean): RegExp;
    regexPatternForAttr(condition: AttributeValue): string;
}
