import { Element } from "@opticss/element-analysis";
import { TemplateAnalysis, TemplateTypes } from "@opticss/template-api";
import { MultiMap } from "@opticss/util";
import * as postcss from "postcss";
import { BSTree, MultiDictionary } from "typescript-collections";
import { ParsedCssFile } from "../../CssFile";
import { OptimizationPass } from "../../OptimizationPass";
import { OptimizationContexts } from "./OptimizationContext";
import { DeclarationInfo, SelectorInfo } from "./StyleInfo";
/**
 * Efficient navigation of the selectors and declarations of a stylesheet
 * and how they intersect at elements.
 */
export declare class DeclarationMapper {
    /**
     * All optimization contexts within which, declarations can be merged.
     */
    contexts: OptimizationContexts;
    /**
     * map of elements to the selectors that match it -- those selectors are in
     * order of lesser to greater precedence. */
    elementDeclarations: Map<Element, MultiDictionary<string, DeclarationInfo>>;
    /** binary search tree of selectors. sorts as they are added and allows in order traversal. */
    selectorTree: BSTree<SelectorInfo>;
    /** Each entry for the declaration represents a selector it was duplicated for,
     * and each decl info in that entry, if more than one, represents a longhand
     * it was expanded into from a shorthand.
     */
    declarationInfos: MultiMap<postcss.Declaration, DeclarationInfo[]>;
    constructor(pass: OptimizationPass, analyses: Array<TemplateAnalysis<keyof TemplateTypes>>, files: Array<ParsedCssFile>);
    private makeDeclInfo;
    private trackDeclarationInfo;
    private addDeclInfoToElements;
}
