import * as postcss from "postcss";
import * as specificity from "specificity";
import { Dictionary, MultiDictionary } from "typescript-collections";
import { ParsedSelector } from "../../parseSelector";
import { RuleScope } from "../../util/cssIntrospection";
import { DeclarationInfo } from "./StyleInfo";
export declare class OptimizationContext {
    /**
     * The key for this optimization context that allows it to be merged across
     * distinct stylesheet regions.
     */
    key: string;
    /** atrule scopes for this context. All are functionally equivalent. */
    scopes: Array<RuleScope>;
    /** runtime selector scoping for this context. */
    selectorContext: ParsedSelector | undefined;
    root: postcss.Root;
    specificity: specificity.Specificity | undefined;
    /**
     * map of property keys to a dictionary of values multi-mapped to
     * the declaration info for this context. The multi-mapped values
     * are in the order of stylesheet precedence.
     */
    declarationMap: Dictionary<string, MultiDictionary<string, DeclarationInfo>>;
    /**
     * Declaration infos that belong to this optimization context;
     */
    declarationInfos: Set<DeclarationInfo>;
    /**
     * A set of properties that are declared within this optimization context.
     */
    authoredProps: Set<string>;
    constructor(key: string, scope: RuleScope, root: postcss.Root, selectorContext: ParsedSelector | undefined);
    getDeclarationValues(prop: string): MultiDictionary<string, DeclarationInfo>;
}
export declare class OptimizationContexts {
    private contexts;
    constructor();
    [Symbol.iterator](): IterableIterator<OptimizationContext>;
    getContext(root: postcss.Root, scope: RuleScope, selectorContext: ParsedSelector | undefined): OptimizationContext;
    getKey(scope: RuleScope, selectorContext: string | undefined): string;
}
