import { TemplateAnalysis, TemplateIntegrationOptions, TemplateTypes } from "@opticss/template-api";
import { MultiMap } from "@opticss/util";
import * as postcss from "postcss";
import { ParsedCssFile } from "../../CssFile";
import { Initializers } from "../../initializers";
import { OptiCSSOptions } from "../../OpticssOptions";
import { OptimizationPass } from "../../OptimizationPass";
import { ParsedSelectorAndRule } from "../../query";
import { MultiFileOptimization } from "../Optimization";
import { DeclarationMapper } from "./DeclarationMapper";
import { OptimizationContext } from "./OptimizationContext";
export declare class MergeDeclarations implements MultiFileOptimization {
    name: string;
    initializers: Array<keyof Initializers>;
    private templateOptions;
    constructor(_options: OptiCSSOptions, templateOptions: TemplateIntegrationOptions);
    optimizeAllFiles(pass: OptimizationPass, analyses: Array<TemplateAnalysis<keyof TemplateTypes>>, files: Array<ParsedCssFile>): void;
    private isMergeable;
    /**
     * Returns all the sets of mergeable declaration sets that exist in the given
     * optimization context. The sets returned from this method do not
     * respect the cascade or take into account whether the merge would be a net
     * benefit.
     */
    private mergeablesForContext;
    private findUnusedAttributes;
    /**
     * Merges a set of mergeable declarations and updates the state of the
     * declaration data accordingly.
     * @param mergeableDeclarations A set of declarations that should be merged.
     * @returns ParsedSelectors that were removed
     */
    private mergeDeclarationSet;
    private checkDupesAndExpandShorthands;
    /**
     * Analyzes declarations that can be merged within the same optimization context
     * and segments them by cascade conflicts.
     *
     * A cascade conflict occurs when the declarations we'd like to merge have an
     * intervening declaration that sets the value to a different value on the same
     * element for at least one runtime state.
     *
     * TODO:
     *   - Move the merged declaration down the cascade if possible to facilitate
     *     a merge.
     *   - Don't consider it a merge conflict if the element analysis indicates
     *     that declarations are mutually exclusive on the same element.
     *   - Some shorthands may merge better (or at all) with one segment than
     *     another.
     *
     * @param actions The current {{OptimizationPass}} actions.
     * @param mapper The declaration map information for the current pass.
     * @param declInfos information about the declarations that can be merged.
     *   These must be in order of lowest to highest precedence.
     * @returns groups of {{DeclarationInfo}}s that should be merged together.
     */
    private segmentByCascadeConflicts;
    removeFullyMergedSelectors(pass: OptimizationPass, mapper: DeclarationMapper, rulesNeedingSelectorPruning: MultiMap<postcss.Rule, [OptimizationContext, postcss.Declaration]>): Array<ParsedSelectorAndRule>;
}
