import { TemplateAnalysis, TemplateIntegrationOptions, TemplateTypes } from "@opticss/template-api";
import { ParsedCssFile } from "../CssFile";
import { Initializers } from "../initializers";
import { OptiCSSOptions } from "../OpticssOptions";
import { OptimizationPass } from "../OptimizationPass";
export interface OptimizationConstructor {
    new (options: OptiCSSOptions, templateOptions: TemplateIntegrationOptions): Optimization;
}
export interface SingleFileOptimization {
    name: string;
    readonly initializers: Array<keyof Initializers>;
    optimizeSingleFile(pass: OptimizationPass, analyses: Array<TemplateAnalysis<keyof TemplateTypes>>, file: ParsedCssFile): void;
}
export interface MultiFileOptimization {
    name: string;
    readonly initializers: Array<keyof Initializers>;
    optimizeAllFiles(pass: OptimizationPass, analyses: Array<TemplateAnalysis<keyof TemplateTypes>>, files: Array<ParsedCssFile>): void;
}
export declare function isMultiFileOptimization(optimization: Optimization): optimization is MultiFileOptimization;
export declare function isSingleFileOptimization(optimization: Optimization): optimization is SingleFileOptimization;
export declare type Optimization = SingleFileOptimization | MultiFileOptimization | SingleFileOptimization & MultiFileOptimization;
