import { TemplateAnalysis, TemplateTypes } from "@opticss/template-api";
import { ParsedCssFile } from "../CssFile";
import { Initializers } from "../initializers";
import { OptiCSSOptions } from "../OpticssOptions";
import { OptimizationPass } from "../OptimizationPass";
import { SingleFileOptimization } from "./Optimization";
/**
 * Removed all rules in a single stylesheet that will never match any Selectables
 * discovered during analysis.
 */
export declare class RemoveUnusedStyles implements SingleFileOptimization {
    name: string;
    initializers: Array<keyof Initializers>;
    /**
     * Create a new instance of this optimizer
     * @param options - The project optimizer's options.
     */
    constructor(_options: OptiCSSOptions);
    /**
     * Provided an OptimizationPass, all TemplateAnalyses, and a ParsedCssFile,
     * remove all unused styles.
     * @param analyses - All TemplateAnalyses found during analysis
     * @param file - The parsed CSS file to optimize.
     */
    optimizeSingleFile(pass: OptimizationPass, analyses: Array<TemplateAnalysis<keyof TemplateTypes>>, file: ParsedCssFile): void;
}
