import { Element } from "@opticss/element-analysis";
import postcss = require("postcss");
import { ParsedSelector } from "./parseSelector";
export interface SelectorQuery {
    execute(container: postcss.Container, selectorFactory?: SelectorFactory): ClassifiedParsedSelectors;
}
export interface ParsedSelectorAndRule {
    parsedSelector: ParsedSelector;
    rule: postcss.Rule;
}
export interface ClassifiedParsedSelectors {
    main: ParsedSelectorAndRule[];
    other: {
        [classification: string]: ParsedSelectorAndRule[];
    };
}
export declare function allParsedSelectors(result: ClassifiedParsedSelectors): ParsedSelector[];
export interface SelectorFactory {
    getParsedSelectors(node: postcss.Rule): ParsedSelector[];
}
export declare class SelectorCache implements SelectorFactory {
    _cache: WeakMap<postcss.Rule, ParsedSelector[]>;
    constructor();
    getParsedSelectors(rule: postcss.Rule): ParsedSelector[];
    clear(): void;
    reset(rule: postcss.Rule): void;
}
/**
 * This query finds all selectors for which the selector references the given
 * target elements in any compound selector.
 *
 * When negated, it finds all selectors that can be proven to never reference
 * any of the target elements.
 */
export declare class QuerySelectorReferences implements SelectorQuery {
    targets: Array<Element>;
    negate: boolean;
    constructor(elements: Array<Element>, negate?: boolean);
    execute(container: postcss.Container, selectorFactory?: SelectorFactory): ClassifiedParsedSelectors;
}
/**
 * This query finds all selectors for which the key selector may match the
 * given element in any of its possible dynamic states.
 *
 * The returned selectors may not actually match depending on the selector
 * context and various combinators.
 */
export declare class QueryKeySelector implements SelectorQuery {
    target: Element;
    constructor(obj: Element);
    execute(container: postcss.Container, selectorFactory?: SelectorFactory): ClassifiedParsedSelectors;
}
