export declare class IdentGenerator {
    lastIdent: string;
    returnedIdents: Array<string>;
    reservedIdents: Set<string>;
    private counters;
    private identChar;
    private increment;
    constructor(caseInsensitive?: boolean);
    nextIdent(): string;
    private generateNextIdent;
    /** An iterator that produces an infinite sequence of identifiers. */
    idents(): IterableIterator<string>;
    /**
     * When a generated ident is no longer in use, it should be returned
     * so it can be re-used.
     */
    returnIdent(ident: string): void;
    reserve(...idents: string[]): void;
    isReserved(ident: string): boolean;
}
export declare class IdentGenerators<Namespace extends string = string> {
    namespaces: {
        [name: string]: IdentGenerator;
    };
    constructor(caseInsensitive: boolean, ...namespaces: Array<Namespace>);
    /**
     * Look up the ident generator for the given namespace. Raise an error if no
     * no such namespace exists.
     */
    get(namespace: Namespace): IdentGenerator;
    /**
     * Generate an ident from the given namespace.
     */
    nextIdent(namespace: Namespace): string;
    /**
     * Return an ident to be generated for the next requested ident.
     */
    returnIdent(namespace: Namespace, ident: string): void;
    /**
     * Reserved idents are never output by the ident generator.
     * @param namespace The namespace that ident should come from.
     * @param idents The ident that should not be generated.
     */
    reserve(namespace: Namespace, ...idents: string[]): void;
}
