import { whatever } from "@opticss/util";
import * as postcss from "postcss";
import * as selectorParser from "postcss-selector-parser";
import { ParsedCssFile } from "../CssFile";
import { ParsedSelector } from "../parseSelector";
import { SelectorCache } from "../query";
export declare type RuleScope = Array<postcss.AtRule>;
export declare type RuleIteratorWithScope = (rule: postcss.Rule, scope: RuleScope) => false | undefined | void;
export declare function walkRules(container: postcss.Container, eachRule: RuleIteratorWithScope): void;
export interface IdentTypes {
    id?: boolean;
    class?: boolean;
}
export declare type IdentNode = selectorParser.Identifier | selectorParser.ClassName;
/**
 * Iterate over all identifiers (ex: .class and #id) in a given selector.
 * @param sel ParsedSelector to walk.
 * @param idents Object of type IdentTypes to indicate idents to traverse over.
 * @param cb Callback function passed the IdentNode.
 */
export declare function eachSelectorIdent(sel: ParsedSelector, idents: IdentTypes, cb: (node: IdentNode) => void): void;
/**
 * Iterate over all identifiers (ex: .class and #id) in a given postcss tree.
 * @param root Postcss root to walk.
 * @param cache SelectorCache to ensure we are working with the same selector instances.
 * @param idents Object of type IdentTypes to indicate idents to traverse over.
 * @param cb Callback function passed the identifier, rule and parsed selector.
 */
export declare function eachIdent(root: postcss.Root, cache: SelectorCache, idents: IdentTypes, cb: (node: IdentNode, rule: postcss.Rule, sel: ParsedSelector) => void): void;
/**
 * Iterate over all identifiers (ex: .class and #id) in a given list of CSS files.
 * @param files List of files to parse.
 * @param cache SelectorCache to ensure we are working with the same selector instances.
 * @param idents Object of type IdentTypes to indicate idents to traverse over.
 * @param cb Callback function passed the identifier, rule and parsed selector.
 */
export declare function eachFileIdent(files: ParsedCssFile[], cache: SelectorCache, idents: IdentTypes, cb: (node: IdentNode, rule: postcss.Rule, sel: ParsedSelector) => void): void;
/**
 * Iterate over all selectors in a list of CSS files.
 * @param files List of ParsedCssFile files to iterate over.
 * @param cache SelectorCache to ensure we are working with the same selector instances.
 * @param cb Callback function passed the rule and parsed selector.
 */
export declare function eachFileSelector(files: ParsedCssFile[], cache: SelectorCache, cb: (rule: postcss.Rule, sel: ParsedSelector) => void): void;
/**
 * Iterate over all selectors in a single postcss tree.
 * @param root Postcss root to walk.
 * @param cache SelectorCache to ensure we are working with the same selector instances.
 * @param cb Callback function passed the rule and parsed selector.
 */
export declare function eachSelector(root: postcss.Root, cache: SelectorCache, cb: (rule: postcss.Rule, sel: ParsedSelector) => void): void;
export declare function isNode(node: whatever): node is postcss.Node;
/**
 * Test if a postcss node is an at-rule.
 * @param node postcss node to test.
 * @returns True or false if node is an at-rule.
 */
export declare function isAtRule(node: whatever): node is postcss.AtRule;
/**
 * Test if a postcss node is a rule.
 * @param node postcss node to test.
 * @returns True or false if node is a rule.
 */
export declare function isRule(node: whatever): node is postcss.Rule;
/**
 * Test if a postcss node is a declaration.
 */
export declare function isDeclaration(node: whatever): node is postcss.Declaration;
/**
 * Test if a postcss node is a container (Root, At-Rule, or Rule).
 * @param node postcss node to test.
 * @returns whether the node is a container.
 */
export declare function isContainer(node: whatever): node is postcss.Container;
